"use strict";

function _typeof(obj) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (obj) { return typeof obj; } : function (obj) { return obj && "function" == typeof Symbol && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }, _typeof(obj); }

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.addArrows = addArrows;
exports.addLinks = addLinks;
exports.asyncSetInterval = asyncSetInterval;
exports.calculatePosition = calculatePosition;
exports.capitalizeFirstLetter = capitalizeFirstLetter;
exports.computeLastPageSizes = computeLastPageSizes;
exports.createFileName = createFileName;
exports.humanFileSize = exports.getSubmitPath = exports.getCategoryColor = void 0;
exports.intersection = intersection;
exports.jsonToString = exports.isPageFullScreen = exports.isFooterHidden = void 0;
exports.legendCreator = legendCreator;
exports.minus = minus;
exports.predictFileType = exports.parseParamWidth = void 0;
exports.sortCompare = sortCompare;

var d3 = _interopRequireWildcard(require("d3-scale"));

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function _getRequireWildcardCache(nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || _typeof(obj) !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function _iterableToArrayLimit(arr, i) { var _i = arr == null ? null : typeof Symbol !== "undefined" && arr[Symbol.iterator] || arr["@@iterator"]; if (_i == null) return; var _arr = []; var _n = true; var _d = false; var _s, _e; try { for (_i = _i.call(arr); !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

// import { submissionApiPath } from './localconf';
var submissionApiPath = 'FIXME-submissionApiPath';

var humanFileSize = function humanFileSize(size) {
  if (typeof size !== 'number') {
    return '';
  }

  var i = size === 0 ? 0 : Math.floor(Math.log(size) / Math.log(1024));
  var sizeStr = (size / Math.pow(1024, i)).toFixed(2) * 1;
  var suffix = ['B', 'KB', 'MB', 'GB', 'TB'][i];
  return "".concat(sizeStr, " ").concat(suffix);
};

exports.humanFileSize = humanFileSize;

var getSubmitPath = function getSubmitPath(project) {
  var path = project.split('-');
  var programName = path[0];
  var projectCode = path.slice(1).join('-');
  return "".concat(submissionApiPath, "/").concat(programName, "/").concat(projectCode);
};

exports.getSubmitPath = getSubmitPath;

var jsonToString = function jsonToString(data) {
  var schema = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

  var replacer = function replacer(key, value) {
    if (value === null) {
      return undefined;
    }

    if (schema[key] === 'number') {
      var castedValue = Number(value);

      if (Number.isNaN(castedValue)) {
        return value;
      }

      return castedValue;
    }

    return value;
  };

  return JSON.stringify(data, replacer, '  ');
};

exports.jsonToString = jsonToString;

var predictFileType = function predictFileType(dirtyData, fileType) {
  var predictType = fileType;
  var jsonType = 'application/json';
  var tsvType = 'text/tab-separated-values';
  var data = dirtyData.trim();

  if (data.indexOf('{') !== -1 || data.indexOf('}') !== -1) {
    return jsonType;
  }

  if (data.indexOf('\t') !== -1) {
    return tsvType;
  }

  return predictType;
};
/**
 * Little wrapper around setinterval with a guard to prevent an async function
 * from being invoked multiple times.
 *
 * @param {()=>Promise} lambda callback should return a Promise
 * @param {int} timeoutMs passed through to setinterval
 * @return the setinterval id (can be passed to clearinterval)
 */


exports.predictFileType = predictFileType;

function asyncSetInterval(_x, _x2) {
  return _asyncSetInterval.apply(this, arguments);
}

function _asyncSetInterval() {
  _asyncSetInterval = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee(lambda, timeoutMs) {
    var isRunningGuard;
    return regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            isRunningGuard = false;
            return _context.abrupt("return", setInterval(function () {
              if (!isRunningGuard) {
                isRunningGuard = true;
                lambda().then(function () {
                  isRunningGuard = false;
                });
              }
            }, timeoutMs));

          case 2:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  }));
  return _asyncSetInterval.apply(this, arguments);
}

var getCategoryColor = function getCategoryColor(category) {
  var colorMap = {
    clinical: '#05B8EE',
    biospecimen: '#27AE60',
    data_file: '#7EC500',
    metadata_file: '#F4B940',
    analysis: '#FF7ABC',
    administrative: '#AD91FF',
    notation: '#E74C3C',
    index_file: '#26D9B1',
    clinical_assessment: '#3283C8',
    medical_history: '#05B8EE',
    satellite: d3.schemeCategory20[11],
    radar: d3.schemeCategory20[16],
    stream_gauge: d3.schemeCategory20[19],
    weather_station: d3.schemeCategory20[10],
    data_observations: d3.schemeCategory20[3],
    experimental_methods: d3.schemeCategory20[4],
    Imaging: d3.schemeCategory20[5],
    study_administration: d3.schemeCategory20[6],
    subject_characteristics: d3.schemeCategory20[7]
  };
  var defaultColor = '#9B9B9B';
  return colorMap[category] ? colorMap[category] : defaultColor;
};

exports.getCategoryColor = getCategoryColor;

function legendCreator(legendGroup, nodes, legendWidth) {
  // Find all unique categories
  var uniqueCategoriesList = nodes.reduce(function (acc, elem) {
    if (acc.indexOf(elem.category) === -1) {
      acc.push(elem.category);
    }

    return acc;
  }, []);
  uniqueCategoriesList.sort(function (aIn, bIn) {
    var a = aIn.toLowerCase();
    var b = bIn.toLowerCase();

    if (a < b) {
      return -1;
    }

    if (a > b) {
      return 1;
    }

    return 0;
  });
  var legendFontSize = '0.9em'; // Make Legend

  legendGroup.selectAll('text').data(uniqueCategoriesList).enter().append('text').attr('x', legendWidth / 2).attr('y', function (d, i) {
    return "".concat(1.5 * (2.5 + i), "em");
  }).attr('text-anchor', 'middle').attr('fill', function (d) {
    return getCategoryColor(d);
  }).style('font-size', legendFontSize).text(function (d) {
    return d;
  });
  legendGroup.append('text').attr('x', legendWidth / 2).attr('y', "".concat(2, "em")).attr('text-anchor', 'middle').text('Categories').style('font-size', legendFontSize).style('text-decoration', 'underline');
}

function addArrows(graphSvg) {
  graphSvg.append('svg:defs').append('svg:marker').attr('id', 'end-arrow').attr('viewBox', '0 -5 10 10').attr('fill', 'darkgray').attr('refX', 0).attr('refY', 0).attr('markerWidth', 6).attr('markerHeight', 6).attr('orient', 'auto').append('svg:path').attr('d', 'M0,-5L10,0L0,5');
}

function addLinks(graphSvg, edges) {
  return graphSvg.append('g').selectAll('path').data(edges).enter().append('path').attr('stroke-width', 2).attr('marker-mid', 'url(#end-arrow)').attr('stroke', 'darkgray').attr('fill', 'none');
}
/**
 * Compute SVG coordinates fx, fy for each node in nodes.
 * Decorate each node with .fx and .fy property as side effect.
 *
 * @param {Array<Node>} nodes each decorated with a position [width,height] in [0,1]
 * @param {*} graphWidth
 * @param {*} graphHeight
 */


function calculatePosition(nodes, graphWidth, graphHeight) {
  // Calculate the appropriate position of each node on the graph
  var fyVals = [];
  var retNodes = nodes;

  for (var i = 0; i < nodes.length; i += 1) {
    retNodes[i].fx = retNodes[i].position[0] * graphWidth;
    retNodes[i].fy = retNodes[i].position[1] * graphHeight;

    if (fyVals.indexOf(retNodes[i].fy) === -1) {
      fyVals.push(retNodes[i].fy);
    }
  }

  return {
    retNodes: retNodes,
    fyValsLength: fyVals.length
  };
}
/**
 * Type agnostic compare thunk for Array.sort
 * @param {*} a
 * @param {*} b
 */


function sortCompare(a, b) {
  if (a === b) {
    return 0;
  }

  return a < b ? -1 : 1;
}

function computeLastPageSizes(filesMap, pageSize) {
  return Object.keys(filesMap).reduce(function (d, key) {
    var result = d;
    result[key] = filesMap[key].length % pageSize;
    return result;
  }, {});
}

function capitalizeFirstLetter(str) {
  var res = str.replace(/_/gi, ' ');
  return res.replace(/\w\S*/g, function (txt) {
    return txt.charAt(0).toUpperCase() + txt.substr(1).toLowerCase();
  });
}
/**
 * Avoid importing underscore just for this ... export for testing
 * @method intersection
 * @param aList {Array<String>}
 * @param bList {Array<String>}
 * @return list of intersecting elements
 */


function intersection(aList, bList) {
  var key2Count = aList.concat(bList).reduce(function (db, it) {
    var res = db;

    if (res[it]) {
      res[it] += 1;
    } else {
      res[it] = 1;
    }

    return res;
  }, {});
  return Object.entries(key2Count).filter(function (kv) {
    return kv[1] > 1;
  }).map(function (_ref) {
    var _ref2 = _slicedToArray(_ref, 1),
        k = _ref2[0];

    return k;
  });
}

function minus(aList, bList) {
  var key2Count = aList.concat(bList).concat(aList).reduce(function (db, it) {
    var res = db;

    if (res[it]) {
      res[it] += 1;
    } else {
      res[it] = 1;
    }

    return res;
  }, {});
  return Object.entries(key2Count).filter(function (kv) {
    return kv[1] === 2;
  }).map(function (_ref3) {
    var _ref4 = _slicedToArray(_ref3, 1),
        k = _ref4[0];

    return k;
  });
}

var parseParamWidth = function parseParamWidth(width) {
  return typeof width === 'number' ? "".concat(width, "px") : width;
};

exports.parseParamWidth = parseParamWidth;

var isPageFullScreen = function isPageFullScreen(pathname) {
  return !!(pathname && (pathname.toLowerCase() === '/dd' || pathname.toLowerCase().startsWith('/dd/') || pathname.toLowerCase() === '/cohort-tools' || pathname.toLowerCase().startsWith('/cohort-tools/')));
};

exports.isPageFullScreen = isPageFullScreen;

var isFooterHidden = function isFooterHidden(pathname) {
  return !!(pathname && (pathname.toLowerCase() === '/dd' || pathname.toLowerCase().startsWith('/dd/')));
};

exports.isFooterHidden = isFooterHidden;

function createFileName(fileName, filePreFix) {
  var date = new Date();
  var yyyy = date.getFullYear();
  var dd = date.getDate();
  var mm = date.getMonth() + 1;

  if (dd < 10) {
    dd = "0".concat(dd);
  }

  if (mm < 10) {
    mm = "0".concat(mm);
  }

  var todaysDate = "".concat(yyyy, "-").concat(mm, "-").concat(dd);
  var hours = date.getHours();
  var minutes = date.getMinutes();
  var seconds = date.getSeconds();

  if (hours < 10) {
    hours = "0".concat(hours);
  }

  if (minutes < 10) {
    minutes = "0".concat(minutes);
  }

  if (seconds < 10) {
    seconds = "0".concat(seconds);
  }

  return "".concat(filePreFix).concat(fileName, " ").concat(todaysDate, " ").concat(hours, "-").concat(minutes, "-").concat(seconds);
}