"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _react = _interopRequireDefault(require("react"));

var _renderer = require("@react-pdf/renderer");

var _RenderSvg = _interopRequireDefault(require("./RenderSvg"));

var _helper = require("../NodeCategories/helper");

var _utils = require("../../utils");

var _util = require("./util");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { _defineProperty(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var styles = _renderer.StyleSheet.create({
  row: {
    margin: 'auto',
    flexDirection: 'row'
  },
  categoryStyle: {
    flexDirection: 'row',
    padding: '7px 0px 7px 10px'
  },
  hr: {
    height: '4px',
    backgroundColor: '#E7E5E5'
  },
  nodeInfo: {
    flexDirection: 'row',
    padding: '6px 15px 2px 15px',
    backgroundColor: '#f4f5f5',
    display: 'flex'
  },
  tagsInfo: {
    flexDirection: 'row',
    alignItems: 'center',
    paddingLeft: '15px',
    paddingBottom: '5px',
    height: '30px',
    // padding: '6px 0px 2px 15px',
    backgroundColor: '#f4f5f5',
    display: 'flex',
    marginBottom: '-12px'
  },
  nodeTitle: {
    color: '#000000',
    fontSize: '10px',
    fontWeight: 'heavy',
    fontFamily: (0, _util.FontRegistry)('NunitoBold'),
    width: '158.5px' // marginRight: '75px',

  },
  nodeDesc: {
    color: '#000000',
    fontSize: '9px',
    paddingTop: '-2px',
    lineHeight: 1.2,
    overflowWrap: 'break-word',
    width: '650px',
    textAlign: 'justify',
    fontFamily: (0, _util.FontRegistry)('NunitoNormal')
  },
  categoryHeader: {
    marginLeft: '10px',
    fontWeight: 'heavy',
    fontSize: '11.25px',
    paddingTop: '8px',
    fontFamily: (0, _util.FontRegistry)('NunitoExtraBold')
  },
  nodeAssignment: {
    // float: 'right',
    paddingTop: '3px',
    paddingRight: '10px',
    paddingLeft: '10px',
    borderRadius: '8px',
    backgroundColor: '#fff',
    border: '0.5px solid #cdcdcd',
    marginRight: '11em'
  },
  nodeClass: {
    paddingTop: '3px',
    paddingRight: '10px',
    paddingLeft: '10px',
    borderRadius: '8px',
    backgroundColor: '#fff',
    border: '0.5px solid #cdcdcd'
  },
  tagContainer: {
    position: 'relative',
    left: '139em',
    display: 'flex',
    flexDirection: 'row'
  },
  label: {
    fontWeight: '900',
    fontSize: '7px',
    paddingTop: '0px',
    paddingBottom: '2px',
    float: 'left',
    color: '#6c6c6c',
    fontFamily: (0, _util.FontRegistry)('NunitoExtraBold')
  },
  assignment: {
    fontSize: '8px',
    paddingTop: '2px',
    marginRight: '10px',
    border: '0.5px solid #cdcdcd',
    borderRadius: '8px',
    textAlign: 'center',
    backgroundColor: '#fff',
    color: '#2982af',
    fontFamily: (0, _util.FontRegistry)('NunitoNormal')
  },
  class: {
    fontSize: '8px',
    paddingRight: '4px',
    paddingTop: '2px',
    border: '0.5px solid #cdcdcd',
    borderRadius: '8px',
    backgroundColor: '#fff',
    color: '#2982af',
    height: '13px',
    fontFamily: (0, _util.FontRegistry)('NunitoNormal')
  }
});

var createStyle = function createStyle(classes, categoryColor) {
  return _objectSpread(_objectSpread({}, classes), {
    borderLeft: "5px solid ".concat(categoryColor)
  });
};

var PdfTitle = function PdfTitle(node) {
  var svgNode = document.querySelector("svg.".concat(node.category));
  var SvgIcon = (0, _RenderSvg.default)(svgNode);
  var categoryColor = (0, _helper.getCategoryColor)(node.category);
  return /*#__PURE__*/_react.default.createElement(_renderer.View, null, /*#__PURE__*/_react.default.createElement(_renderer.View, {
    style: createStyle(styles.categoryStyle, categoryColor)
  }, SvgIcon, /*#__PURE__*/_react.default.createElement(_renderer.Text, {
    style: _objectSpread({
      color: categoryColor
    }, styles.categoryHeader)
  }, (0, _utils.capitalizeFirstLetter)(node.category))), /*#__PURE__*/_react.default.createElement(_renderer.View, {
    style: createStyle(styles.hr, categoryColor)
  }), /*#__PURE__*/_react.default.createElement(_renderer.View, {
    style: createStyle(styles.nodeInfo, categoryColor)
  }, /*#__PURE__*/_react.default.createElement(_renderer.Text, {
    style: styles.nodeTitle
  }, (0, _utils.capitalizeFirstLetter)(node.title)), /*#__PURE__*/_react.default.createElement(_renderer.Text, {
    style: styles.nodeDesc
  }, node.desc)), /*#__PURE__*/_react.default.createElement(_renderer.View, {
    style: createStyle(styles.tagsInfo, categoryColor)
  }, /*#__PURE__*/_react.default.createElement("span", {
    style: styles.tagContainer
  }, node.assignment ? /*#__PURE__*/_react.default.createElement(_renderer.Text, {
    style: styles.nodeAssignment
  }, /*#__PURE__*/_react.default.createElement("span", {
    style: styles.label
  }, 'Assignment: '), /*#__PURE__*/_react.default.createElement("span", {
    style: styles.assignment
  }, (0, _utils.capitalizeFirstLetter)(node.assignment))) : /*#__PURE__*/_react.default.createElement(_renderer.Text, null), node.nodeClass ? /*#__PURE__*/_react.default.createElement(_renderer.Text, {
    style: styles.nodeClass
  }, /*#__PURE__*/_react.default.createElement("span", {
    style: styles.label
  }, 'Class: '), /*#__PURE__*/_react.default.createElement("span", {
    style: styles.class
  }, (0, _utils.capitalizeFirstLetter)(node.nodeClass))) : /*#__PURE__*/_react.default.createElement(_renderer.Text, null))));
};

var _default = PdfTitle;
exports.default = _default;