"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _react = _interopRequireDefault(require("react"));

var _fileSaver = require("file-saver");

var _core = require("@material-ui/core");

var _renderer = require("@react-pdf/renderer");

var _bentoComponents = require("bento-components");

var _Pdf = _interopRequireDefault(require("./Pdf"));

var _Pdf2 = _interopRequireDefault(require("../LandscapeNodePDF/Pdf"));

var _fileManifestConfig = require("../../../../config/file-manifest-config");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function _iterableToArrayLimit(arr, i) { var _i = arr == null ? null : typeof Symbol !== "undefined" && arr[Symbol.iterator] || arr["@@iterator"]; if (_i == null) return; var _arr = []; var _n = true; var _d = false; var _s, _e; try { for (_i = _i.call(arr); !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

var DownloadButton = function DownloadButton(_ref) {
  var classes = _ref.classes,
      config = _ref.config,
      documentData = _ref.documentData,
      fileName = _ref.fileName,
      isFileManifest = _ref.isFileManifest;

  var _React$useState = _react.default.useState(false),
      _React$useState2 = _slicedToArray(_React$useState, 2),
      isLoading = _React$useState2[0],
      setLoading = _React$useState2[1];

  var theme = (0, _core.createTheme)({
    overrides: {
      MuiButton: {
        root: {
          minWidth: '35px',
          paddingRight: '10px',
          '&:hover': {
            backgroundColor: 'none'
          }
        }
      }
    }
  });

  var tsvMiddleware = function tsvMiddleware(node) {
    var line = 'type';
    var links = node.links;

    if (links && links.length) {
      links.forEach(function (c) {
        if (c.targetId && String(c.generatedType).toLowerCase() !== 'loader-generated') {
          line += '\t'.concat(" ", c.target_type, ".").concat(c.targetId);
        }
      });
    }

    return line;
  };

  var generateFileManifest = function generateFileManifest(node) {
    var line = tsvMiddleware(node);
    var arr = Object.entries(node.properties);
    var mergedArr = arr.concat(_fileManifestConfig.fileManifestDownload);
    mergedArr.forEach(function (_ref2) {
      var _ref3 = _slicedToArray(_ref2, 2),
          key = _ref3[0],
          value = _ref3[1];

      if (value.src !== 'Loader-derived') {
        line += '\t'.concat("".concat(key));
      }
    });
    var text = "".concat(line, "\r\n").concat(node.title);
    return text;
  };

  var convertToTSV = function convertToTSV(node) {
    var line = tsvMiddleware(node);
    Object.keys(node.properties).forEach(function (key) {
      line += '\t'.concat("".concat(key));
    });
    var text = "".concat(line, "\r\n").concat(node.title);
    return text;
  };

  var generatePdfDocument = /*#__PURE__*/function () {
    var _ref4 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee(object) {
      var document, blob;
      return regeneratorRuntime.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              document = config.type === 'document' ? object : [object];
              _context.next = 3;
              return (0, _renderer.pdf)(config.landscape ? /*#__PURE__*/_react.default.createElement(_Pdf2.default, {
                nodes: document
              }) : /*#__PURE__*/_react.default.createElement(_Pdf.default, {
                nodes: document
              })).toBlob();

            case 3:
              blob = _context.sent;
              setLoading(false);
              (0, _fileSaver.saveAs)(blob, "".concat(fileName, ".pdf"));

            case 6:
            case "end":
              return _context.stop();
          }
        }
      }, _callee);
    }));

    return function generatePdfDocument(_x) {
      return _ref4.apply(this, arguments);
    };
  }();

  var downloadPdf = function downloadPdf() {
    setLoading(true);
    setTimeout(function () {
      generatePdfDocument(documentData);
    }, 50);
  };

  var downloadTsv = function downloadTsv() {
    var tsv = isFileManifest ? generateFileManifest(documentData) : convertToTSV(documentData);
    var exportData = new Blob([tsv], {
      type: 'data:text/tab-separated-values'
    });
    (0, _fileSaver.saveAs)(exportData, "".concat(fileName, ".tsv"));
  };

  var download = function download() {
    if (config.fileType === 'tsv') {
      downloadTsv();
    } else {
      downloadPdf();
    }
  };

  var btnClass = config.type === 'document' ? (0, _bentoComponents.cn)(classes.headerButton, classes.documentDownloadBtn) : (0, _bentoComponents.cn)(classes.headerButton, classes.nodeDownloadBtn);
  return /*#__PURE__*/_react.default.createElement(_core.MuiThemeProvider, {
    theme: theme
  }, /*#__PURE__*/_react.default.createElement(_core.Button, {
    type: "button",
    disableRipple: true // className={isLoading ? config.loading : config.class}
    ,
    className: btnClass,
    onClick: function onClick() {
      return download();
    }
  }, isLoading ? ' Loading document... ' : config.image ? /*#__PURE__*/_react.default.createElement("img", {
    className: classes.image,
    src: config.image,
    alt: "download pdf"
  }) : 'DOWNLOAD DICTIONARY'));
};

var styles = function styles() {
  return {
    root: {
      fontFamily: '"Open Sans", sans-serif',
      fontSize: '9px',
      letterSpacing: '0.025em',
      color: '#000'
    },
    headerButton: {
      padding: 0
    },
    documentDownloadBtn: {
      background: '#0D71A3',
      paddingRight: '15px',
      paddingLeft: '20px',
      color: '#fff',
      fontSize: '11px',
      marginBottom: '15px',
      height: '38px',
      marginRight: '21px',
      '&:hover': {
        textDecoration: 'none'
      }
    },
    nodeDownloadBtn: {
      marginLeft: '15px',
      background: '#0D71A3',
      height: '35px',
      color: '#fff'
    },
    image: {
      width: '35px'
    },
    loading: {}
  };
};

var _default = (0, _core.withStyles)(styles)(DownloadButton);

exports.default = _default;